# =====================================================================
#  Written by and copyright Carlo Strozzi <carlos@linux.it>.
#
#  mkarray.awk: table-to-array filter for CSA.
#  Copyright (C) 2001-2002 Carlo Strozzi <carlos@linux.it>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# Notes:
#
# Takes a single-record table on stdin and builds an equivalent list
# af AWK array assignments to stdout, enclosed in a BEGIN{} block.
# The special AWK characters " and \ in column values are escaped by
# prepending them with a backslash (\), as the final output is to be
# eval'ed again by AWK.
#
# ChangeLog:
#
# * Development code
#   2002-09-13 Carlo Strozzi <carlos@linux.it>
#
# =====================================================================

BEGIN {

   # Set default array name.
   if (s_array_name == NULL) s_array_name = "a_mk_array"

   print "BEGIN {"
}

NR == 1 { split($0, a_column) }			# Column names.
NR == 2 { next }				# Dashline.

END {
   if (NR > 2) {				# Only if there is data.

      for (s_count = 1; s_count <= NF; s_count++) {

	 # Escape backslashes (\) and double-quotes (") in column values,
	 # then print the array assignment to stdout.

	 s_string = csaMawkBug($s_count)
	 gsub(/"/, "\\\"", s_string)
	 print "  " s_array_name "[\"" a_column[s_count] "\"] = \"" s_string "\""
      }
   }
   print "}"				# Close the output BEGIN{} block.
}

# =====================================================================
# End of mkarray.awk
# =====================================================================
