# =====================================================================
# xmlrpc-format.awk: removes line-breaks from an XML-RPC stream and
# optionally formats it into a visually-comfortable multi-line format,
# useful for debugging. If option '-l' is specified, sets the HTTP
# Content-Length header on replies from servers that do not set it.
#
# Copyright (c) 2003 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: xmlrpc-format.awk,v 1.3 2003/05/07 09:11:26 carlo Exp $

# =====================================================================
# Notes:
# 
# The 'http' option is meant to handle XML-RPC HTTP response headers
# and to pass them along unchanged. This should work in most cases,
# except if the HTTP header contains '<' characters, that this script
# uses as XML element separators.
# =====================================================================

BEGIN {

  # Process options.
  if (http != 1) http = 0

  if (format != 1) fmt = 0
  else fmt = 1

  if (setlength != 1) len = 0
  else len = 1

  # Don't do content-length processing if not HTTP.
  if (!http) len = 0

  RS = "<"
}

http && NR == 1 {

	if (len) {
	   sub(/\n+$/,"\n",$0)
	   printf("%sContent-Length: ", $0)
	}
	else printf("%s", $0)
	next
}

{
	crush = 1
	delay = 0
}

fmt && /^nil *\/ *>/					{ delay = 1 }
fmt && /^(n|stri|v|(\/?(b|d|i)|\/(a|str)))[^>]+>/	{ delay = 2 }

# Override delay for selected events.
fmt && /^\/?data>/					{ delay = 0 }

fmt && /^methodName>/					{ delay = 2 }

/^(b|stri)[^>]+>/					{ crush = 0 }

/^[ \n\t]+$/						{ next }

{
	# Crush the stream.

	if (crush) sub(/>[ \n\t]+$/,">")
}

fmt && /^\/methodResponse>$/			{ $0 = $0 "\n" }

NR > 1	{
		if (len) {
		   out[++x] = "<" $0
		   size += (length($0) + 1)
		}
		else printf("<%s", $0)

		if (fmt) {
		   if (delay) delay--
		   if (!delay && $1 != "/methodResponse>") {
		      if (len) {
			 out[++x] = "\n"
			 size++
		      }
		      else printf("\n")
		   }
		}
}

END	{
		if (NR > 1 && !fmt) {
		   if (len) {
		      out[++x] = "\n"
		      size++
		   }
		   else printf("\n")
		}

		if (len) {
		   # Note the HTTP header separator newline here.
		   printf("%d\n\n",size)
		   while (++y <= x) printf("%s", out[y])
		}
}

# EOF
