# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2002,2003 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _isoencode.awk,v 1.1 2004/08/08 21:15:38 carlo Exp $
# =====================================================================
# string _isoencode(string s)
#
# Encode special HTML and form characters into the equivalent ISO
# entities. The set of characters that are ISO-encoded is:
#
#		& # < > | " ' ` \ \n \t \f SPACE
#
# Warning: '&' must be escaped first, then '#'. They need to be escaped
# to prevent the data from containing statements (both numeric and
# literal) that could be parsed by a Web server as valid SSI calls, or
# ampersand-escaped sequences that would act as formatting instructions
# to the Web browser. See http://www.apache.org/info/css-security/ for
# more info on the so-called Cross-Site Scripting vulnerabilities.
# =====================================================================

function _isoencode(string) {

   gsub(/&/, "\\&amp;", string)			# Ampersand
   gsub(/#/, "\\&#35;", string)			# Hash mark
   gsub(/</, "\\&#60;", string)			# Open tag
   gsub(/>/, "\\&#62;", string)			# Close tag
   gsub(/\|/, "\\&#124;", string)		# Generic separator
   gsub(/"/, "\\&#34;", string)			# double quote
   gsub(/'/, "\\&#39;", string)			# single quote
   gsub(/`/, "\\&#96;", string)			# backtick
   # gsub(/\001/, "\\&#1;", string)		# rc(1)'s ^A
   gsub(/\n/, "\\&#10;", string)		# newline
   gsub(/\t/, "\\&#9;", string)			# tab
   gsub(/\f/, "\\&#12;", string)		# form-feed
   gsub(/ /, "\\&#32;", string)			# space
   gsub(/\\/, "\\&#92;", string)		# backslash

   return string
}

